<?php
/******************************************************************************
 *
 *  LOCAL STORE
 *
 *  local version of Config-Store
 *
 *****************************************************************************/

class LocalStore
{
  //---------------------------------------------------------------------------
  //  PRIVATE VARIABLES
  //
  private $m_configStore;
  private $m_devTree;
  private $m_rootEntityId = null;

  //---------------------------------------------------------------------------
  //  CONSTRUCT
  //
  public function __construct(  )
  {
    $this->m_configStore = array( );
    $string = file_get_contents( DATA_PATH . "confstore/devtree.json" );
    $this->m_devTree = json_decode( $string, true );
    $this->_LoadChildren( $this->m_devTree['devtree'] );
    printf( "Loaded %d entities.\r\n", $this->Size( ) );
  }

  //---------------------------------------------------------------------------
  //  HELPERS
  //

  protected function _LoadChildren( $branch )
  {
    foreach( $branch as $entityId => $children )  {
      if( !( $this->m_rootEntityId ) )  {
        $this->m_rootEntityId = $entityId;
      }

      $this->_LoadEntity( $entityId );
      if( key_exists( 'children', $children ) && is_array( $children['children'] ) )  {
        $this->_LoadChildren( $children['children'] );
      }
    }
  }

  protected function _LoadEntity( $entityId )
  {

    if( file_exists( DATA_PATH . "confstore/" . $entityId . ".json" ) )  {
      //printf( "Load entity( '%s' )\r\n", $entityId );
      $string = file_get_contents( DATA_PATH . "confstore/" . $entityId . ".json" );
      $this->m_configStore[ $entityId ] = json_decode( $string, true );
    }
    else  {
      printf( "NOT FOUND: entity( '%s' )\r\n", $entityId );
    }

  }

  //---------------------------------------------------------------------------
  //  METHODS
  //

  public function Size()
  {
    if( is_array( $this->m_configStore ) )  {
      return count( $this->m_configStore );
    }
    return -1;
  }

  public function Get( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      return $this->m_configStore[ $entityId ];
    }
    return null;
  }

  public function GetRootId()
  {
    return $this->m_rootEntityId;
  }

  public function GetEntityIdAt( $index )
  {
    $keyList = array_keys( $this->m_configStore );
    return $keyList[ $index ];
  }

  public function GetEntityIdList()
  {
    return array_keys( $this->m_configStore );    
  }

  public function Set( $entityId, $entity )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      $this->m_configStore[ $entityId ] = $entity;
      $jsonString = json_encode( $entity, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
      file_put_contents( DATA_PATH . "confstore/" . $entityId . ".json",  $jsonString );
      return true;
    }
    return false;
  }

  public function HasPassword( $entityId  )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      return file_exists( DATA_PATH . "confstore/" . $entityId . ".pass.json" );
    }
    return false;
  }

  public function GetBuild( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore )
     && key_exists( 'adminUi', $this->m_configStore[ $entityId ] )
     && key_exists( 'build', $this->m_configStore[ $entityId ]['adminUi'] )
      )  {
      return intval(  $this->m_configStore[ $entityId ]['adminUi']['build'] );
    }
    return 0;
  }

  public function GetEntityClass( $entityId )
  {
    if( key_exists( $entityId, $this->m_configStore ) )  {
      return $this->m_configStore[ $entityId ]['class'];
    }
    return "";
  }

  public function AddPermissionRecord( $entity, $permission, $filter )
  {
    if( key_exists( 'permission', $entity ) )  {
        
      $permissionHolders = Array( );
      $isInstalled  = false;

      for( $i = 0; $i < count( $entity['permission'] ); $i++ )  {
        $holder = $entity['permission'][ $i ]['w'];
        $permissionHolders[ $holder ] = true;
        if( $entity['permission'][ $i ]['f'] == $filter )  {
          $isInstalled = true;
          break;
        }
      }
        
      if( !$isInstalled )  {
        foreach( $permissionHolders AS $holder => $value )  {
          array_push( $entity['permission'], Array(
            'p' => $permission,
            'w' => $holder,
            'f' => $filter
          ) );
        }
      }

      return $entity;
    }
    
  }

};
?>